library ptwebserver;

import 'dart:io';

class ptConfig {
  ptConfig() {
    hosts = new List<ptHost>();
  }
  String servername;
  List<ptHost> hosts;
  
  String logPath;
  
  
  Map toJson() {
    Map ret = new Map();
    ret['servername']=servername;
    ret['hosts']=hosts;
    return ret;
  }
}

class ptHost {
  ptHost({List<String> pub_index_files}) {
    if(pub_index_files==null) {
      public_index_files = new List<String>();
    } else {
      public_index_files = pub_index_files;
    }
  }
  // Name of the host, mostly for logging
  String name;
  
  // Address to listen on
  String address;
  
  // Hostname to check against
  String hostname;
  
  // Port to check on
  int port;
  
  // Path to the html publicly available files
  String public_path;
  
  // When a directory is hit, if there is no specific file to open, check against this
  List<String> public_index_files;
  
  // Allow directory structure
  bool allow_dir_view;
  
  Map toJson() {
    Map ret = new Map();
    ret['name']=name;
    ret['address']=address;
    ret['hostname']=hostname;
    ret['port']=port;
    ret['public_path']=public_path;
    ret['public_index_files'] = public_index_files;
    ret['allow_dir_view']=allow_dir_view;
    return ret;
  }
}

class ptWebServer {
  ptConfig config;
  
  ptWebServer({String configFile,ptConfig this.config}) {
    
  }
  
  void Run() {
    print("Starting ${config.servername}");
    Map hosts = new Map();
    
    if(config == null) {
      print("Unable to start, please provide a config");
      return;
    }
    
    for(int i=0; i<config.hosts.length; ++i) {
      print("Reading host ${i+1} of ${config.hosts.length}");
      String key = "${config.hosts[i].address}:${config.hosts[i].port}";
      if(!hosts.containsKey(key)) {
        hosts[key]=new List<ptHost>();
      }
      List<ptHost> list = hosts[key];
      list.add(config.hosts[i]);
    }
    
    hosts.forEach((String addrport, List<ptHost> hostList) {
      String addr = hostList[0].address;
      int port = hostList[0].port;
      StartListener(addr,port,hostList);
    });
  }
  
  void StartListener(String address,int port, List<ptHost> hosts) {
    print("Binding to ${address}:${port}");
    
    HttpServer.bind(address,port).then((server) {
      
      print("Listening to ${address}:${port}");
      
      server.listen((HttpRequest request) {
        print("Request from : ${request.connectionInfo.remoteAddress}");
        HttpHeaders headers = request.headers;
        HttpResponse response = request.response;
        headers.forEach((name,List<String> value) {
          for(int i=0; i<value.length; ++i)
            response.write("${name} ${value[i]}\r\n");
        });
        response.close();
      });
    });
  }
}